<?php

namespace Drp\LdapAuthentication;

use Adldap\Models\User;
use Illuminate\Support\Collection;

/**
 * Finds the best group for a given AdLdap user.
 * @package Drp\LdapAuthentication
 */
class GroupFinder
{
    /**
     * Groups users have to be a member of.
     * @var Collection
     */
    protected $allowedGroups;

    /**
     * When checking if the user has the correct groups, whether to check
     * that they exists in 1 or all of them.
     * @var boolean
     */
    protected $inclusiveGroups;

    /**
     * A super user must be a member to both super and standard groups.
     * @var boolean
     */
    protected $needsPrevious;

    /**
     * Constructor
     * @param array $parameters Parameters to parse.
     */
    public function __construct(array $parameters = [])
    {
        $parameters = collect($parameters);
        $this->parseParameters($parameters);
    }

    /**
     * Parses the parameters and set the properties accordingly
     * @param Collection $parameters Collection of parameters to set.
     * @return void
     */
    protected function parseParameters(Collection $parameters)
    {
        $properties = collect(array_keys(get_object_vars($this)));

        $properties->each(function ($property) use ($parameters) {
            $propValue = $parameters->get($property);

            if ($propValue === null) {
                $propValue = config('adldap.' . snake_case($property));
            }

            $setMethodName = 'set' . ucwords($property);

            if (method_exists($this, $setMethodName)) {
                $this->$setMethodName($propValue);
                return;
            }

            $this->$property = $propValue;
        });
    }

    /**
     * Checks if the user is within the groups.
     * @param User $user User to find the group of.
     * @return boolean
     */
    public function find(User $user)
    {
        $userGroup = null;

        foreach ($this->allowedGroups->reverse() as $groupName => $ldapGroups) {
            $ldapGroups = $this->convertIntoCollection($ldapGroups);
            $validGroup = $this->checkGroups($ldapGroups, $user);

            if ($validGroup === false && $this->needsPrevious) {
                break;
            }

            if ($validGroup) {
                $userGroup = $groupName;
            }
        }

        return $userGroup;
    }

    /**
     * Checks if the given user is a member of some or all the groups given.
     * @param Collection $groups Groups to check against.
     * @param User $user User to check with.
     * @return boolean
     */
    protected function checkGroups(Collection $groups, User $user)
    {
        //If no groups have been set, skip the check
        if ($groups->isEmpty()) {
            return true;
        }

        $groupsNotIn = $groups->reject(function ($group) use ($user) {
            // Check with LDAP if this user has group needed on LDAP
            return $user->inGroup($group);
        });

        if ($groupsNotIn->isEmpty() && $this->inclusiveGroups) {
            return true;
        }

        if ($groupsNotIn->count() < $groups->count() && !$this->inclusiveGroups) {
            return true;
        }

        return false;
    }

    /**
     * Sets the allowed groups
     * @param mixed $allowedGroups The string, array or collection to set.
     * @return static
     */
    public function setAllowedGroups($allowedGroups)
    {
        $allowedGroups = $this->convertIntoCollection($allowedGroups);

        $this->allowedGroups = $allowedGroups;

        return $this;
    }

    /**
     * Converts the given parameter into a collection
     * @param mixed $value The value to convert to a collection.
     * @return Collection
     */
    protected function convertIntoCollection($value)
    {
        if (is_array($value) === false &&
            !$value instanceof Collection
        ) {
            $value = (array) $value;
        }

        if (is_array($value)) {
            $value = collect($value);
        }

        return $value;
    }

    /**
     * Sets the inclusiveGroups to use.
     * @param boolean $inclusiveGroups Whether to set the finder as inclusive.
     * @return static
     */
    public function setInclusiveGroups($inclusiveGroups)
    {
        $this->inclusiveGroups = (boolean) $inclusiveGroups;

        return $this;
    }

    /**
     * Sets the needsPrevious to use.
     * @param boolean $needsPrevious Whether to set the finder as needing the previous
     * @return static
     */
    public function setNeedsPrevious($needsPrevious)
    {
        $this->needsPrevious = (boolean) $needsPrevious;

        return $this;
    }
}
